import { Directive, ElementRef, Input, NgModule } from '@angular/core';
import { fromEvent } from 'rxjs';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const objectProto = Object.prototype;
/** @type {?} */
const hasOwnProperty = objectProto.hasOwnProperty;
/**
 * @param {?} object
 * @param {...?} sources
 * @return {?}
 */
function defaults(object, ...sources) {
    object = Object(object);
    sources.forEach((/**
     * @param {?} source
     * @return {?}
     */
    (source) => {
        if (source != null) {
            source = Object(source);
            for (const key in source) {
                if (source.hasOwnProperty(key)) {
                    /** @type {?} */
                    const value = object[key];
                    if (value === undefined ||
                        (eq(value, objectProto[key]) && !hasOwnProperty.call(object, key))) {
                        object[key] = source[key];
                    }
                }
            }
        }
    }));
    return object;
}
/**
 * @param {?} value
 * @param {?} other
 * @return {?}
 */
function eq(value, other) {
    return value === other || (value !== value && other !== other);
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const defaultOptions = {
    defer: false,
    onlyWords: false,
    mask: false,
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class SplitNodes {
    /**
     * @param {?} textContent
     * @param {?} el
     * @param {?} options
     */
    constructor(textContent, el, options) {
        this.textContent = textContent;
        this.el = el;
        this.options = options;
        this.words = [];
        this.chars = [];
        this.lineWords = [];
        this.lineChars = [];
        this.wordsArray = [];
        this.nodes = [];
        this.nodeTypes = {
            ELEMENT_NODE: 1,
            ATTRIBUTE_NODE: 2,
            TEXT_NODE: 3,
            CDATA_SECTION_NODE: 4,
            ENTITY_REFERENCE_NODE: 5,
            ENTITY_NODE: 6,
            PROCESSING_INSTRUCTION_NODE: 7,
            COMMENT_NODE: 8,
            DOCUMENT_NODE: 9,
            DOCUMENT_TYPE_NODE: 10,
            DOCUMENT_FRAGMENT_NODE: 11,
            NOTATION_NODE: 12
        };
    }
    /**
     * @return {?}
     */
    initSplitNodes() {
        this.splitNodes(this.el);
        this.initNewNodes();
        this.setLines();
        this.updateOnResize();
    }
    /**
     * @param {?} el
     * @return {?}
     */
    splitNodes(el) {
        for (let i = 0; i < el.childNodes.length; i++) {
            /** @type {?} */
            const node = el.childNodes[i];
            /** @type {?} */
            const tag = [this.nodeTypes.ELEMENT_NODE, this.nodeTypes.DOCUMENT_NODE, this.nodeTypes.DOCUMENT_FRAGMENT_NODE].indexOf(node.nodeType) !== -1;
            /** @type {?} */
            const text = [this.nodeTypes.TEXT_NODE, this.nodeTypes.CDATA_SECTION_NODE].indexOf(node.nodeType) !== -1;
            if (tag) {
                this.splitNodes(node);
            }
            else if (text) {
                /** @type {?} */
                const words = this.splitNodesIntoWords(node);
                /** @type {?} */
                const wordsArray = [];
                words.forEach((/**
                 * @param {?} chars
                 * @return {?}
                 */
                chars => {
                    /** @type {?} */
                    const wordSpan = document.createElement('span');
                    wordSpan.classList.add('split-text-word');
                    wordSpan.style.display = 'inline-block';
                    wordSpan.style.textIndent = '0';
                    if (this.options.onlyWords) {
                        wordSpan.innerHTML = chars.join('');
                    }
                    else {
                        chars.forEach((/**
                         * @param {?} char
                         * @return {?}
                         */
                        char => {
                            /** @type {?} */
                            const charSpan = document.createElement('span');
                            charSpan.classList.add('split-text-char');
                            charSpan.style.display = 'inherit';
                            charSpan.style.textIndent = '0';
                            charSpan.innerHTML = char;
                            wordSpan.appendChild(charSpan);
                            this.chars.push(charSpan);
                        }));
                    }
                    wordsArray.push(wordSpan);
                    this.words.push(wordSpan);
                }));
                this.wordsArray.push(wordsArray);
                this.nodes.push(node);
            }
        }
    }
    /**
     * @return {?}
     */
    initNewNodes() {
        this.nodes.forEach((/**
         * @param {?} node
         * @param {?} index
         * @return {?}
         */
        (node, index) => {
            this.wordsArray[index].forEach((/**
             * @param {?} word
             * @param {?} idx
             * @return {?}
             */
            (word, idx) => {
                /** @type {?} */
                let maskSpan;
                if (this.options.mask) {
                    maskSpan = document.createElement('span');
                    maskSpan.classList.add('split-text-mask');
                    maskSpan.style.display = 'inline-block';
                    maskSpan.style.overflow = 'hidden';
                    maskSpan.style.textIndent = '0';
                    maskSpan.style.verticalAlign = 'top';
                    maskSpan.appendChild(word);
                    node.parentNode.insertBefore(maskSpan, node);
                }
                else {
                    node.parentNode.insertBefore(word, node);
                }
                /** @type {?} */
                const spaceSpan = document.createElement('span');
                spaceSpan.classList.add('split-text-space');
                spaceSpan.style.display = 'inline';
                spaceSpan.innerHTML = ' ';
                if (this.options.mask) {
                    node.parentNode.insertBefore(spaceSpan, maskSpan.nextSibling);
                }
                else {
                    node.parentNode.insertBefore(spaceSpan, word.nextSibling);
                }
                if (idx === this.wordsArray[index].length - 1) {
                    node.remove();
                }
            }));
        }));
    }
    /**
     * @param {?} word
     * @return {?}
     */
    splitNodesIntoWords(word) {
        /** @type {?} */
        const words = word.textContent.split(' ');
        if (words[0] === '') {
            words.splice(0, 1);
        }
        if (words[words.length - 1] === '') {
            words.splice(words.length - 1, 1);
        }
        return words.map((/**
         * @param {?} item
         * @return {?}
         */
        item => item.split('')));
    }
    /**
     * @param {?} elements
     * @return {?}
     */
    getLines(elements) {
        /** @type {?} */
        const lineElements = [];
        /** @type {?} */
        let line = [];
        /** @type {?} */
        let lineIndex = 0;
        elements.forEach((/**
         * @param {?} el
         * @param {?} index
         * @return {?}
         */
        (el, index) => {
            /** @type {?} */
            const firstElTop = elements[lineIndex].getBoundingClientRect().top;
            /** @type {?} */
            const lastElIndex = elements.length - 1;
            if (el.getBoundingClientRect().top === firstElTop) {
                line.push(el);
                if (index === lastElIndex) {
                    lineElements.push(line);
                }
            }
            else {
                lineElements.push(line);
                lineIndex = index;
                line = [];
                line.push(el);
            }
        }));
        return lineElements;
    }
    /**
     * @return {?}
     */
    updateOnResize() {
        fromEvent(window, 'resize')
            .subscribe((/**
         * @return {?}
         */
        () => {
            this.setLines();
        }));
    }
    /**
     * @return {?}
     */
    setLines() {
        this.lineWords = this.getLines(this.words);
        if (!this.options.onlyWords) {
            this.lineChars = this.getLines(this.chars);
        }
    }
}
if (false) {
    /** @type {?} */
    SplitNodes.prototype.words;
    /** @type {?} */
    SplitNodes.prototype.chars;
    /** @type {?} */
    SplitNodes.prototype.lineWords;
    /** @type {?} */
    SplitNodes.prototype.lineChars;
    /** @type {?} */
    SplitNodes.prototype.wordsArray;
    /** @type {?} */
    SplitNodes.prototype.nodes;
    /** @type {?} */
    SplitNodes.prototype.nodeTypes;
    /**
     * @type {?}
     * @private
     */
    SplitNodes.prototype.textContent;
    /**
     * @type {?}
     * @private
     */
    SplitNodes.prototype.el;
    /**
     * @type {?}
     * @private
     */
    SplitNodes.prototype.options;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NgGxSplitTextDirective {
    /**
     * @param {?} el
     */
    constructor(el) {
        this.el = el;
        this.init = false;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.setCurrentOptions();
        this.initSplitNodes();
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        if (!this.currentOptions.defer) {
            this.initSplit();
        }
    }
    /**
     * @return {?}
     */
    initSplit() {
        if (this.init) {
            console.warn('Warning! Text is already initialized');
            return;
        }
        this.init = true;
        this.saveSrcText();
        this.splitNodes.initSplitNodes();
    }
    /**
     * @private
     * @return {?}
     */
    saveSrcText() {
        this.srcTextContent = this.el.nativeElement.innerHTML;
    }
    /**
     * @return {?}
     */
    get words() {
        return this.splitNodes.words;
    }
    /**
     * @return {?}
     */
    get lineWords() {
        return this.splitNodes.lineWords;
    }
    /**
     * @return {?}
     */
    get chars() {
        if (this.options.onlyWords) {
            console.warn('Warning! You chose only words.');
            return;
        }
        return this.splitNodes.chars;
    }
    /**
     * @return {?}
     */
    get lineChars() {
        if (this.options.onlyWords) {
            console.warn('Warning! You chose only words.');
            return;
        }
        return this.splitNodes.lineChars;
    }
    /**
     * @return {?}
     */
    get nativeElement() {
        return this.el.nativeElement;
    }
    /**
     * @return {?}
     */
    get srcText() {
        return this.srcTextContent;
    }
    /**
     * @return {?}
     */
    get isInit() {
        return this.init;
    }
    /**
     * @return {?}
     */
    resetSplit() {
        this.nativeElement.innerHTML = this.srcText;
    }
    /**
     * @private
     * @return {?}
     */
    setCurrentOptions() {
        this.currentOptions = defaults(this.options, defaultOptions);
    }
    /**
     * @private
     * @return {?}
     */
    initSplitNodes() {
        this.splitNodes = new SplitNodes(this.el.nativeElement.textContent, this.el.nativeElement, this.currentOptions);
    }
}
NgGxSplitTextDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ngGxSplitText], ngGxSplitText'
            },] }
];
/** @nocollapse */
NgGxSplitTextDirective.ctorParameters = () => [
    { type: ElementRef }
];
NgGxSplitTextDirective.propDecorators = {
    options: [{ type: Input, args: ['ngGxSplitText',] }]
};
if (false) {
    /** @type {?} */
    NgGxSplitTextDirective.prototype.options;
    /**
     * @type {?}
     * @private
     */
    NgGxSplitTextDirective.prototype.srcTextContent;
    /**
     * @type {?}
     * @private
     */
    NgGxSplitTextDirective.prototype.init;
    /**
     * @type {?}
     * @private
     */
    NgGxSplitTextDirective.prototype.currentOptions;
    /**
     * @type {?}
     * @private
     */
    NgGxSplitTextDirective.prototype.splitNodes;
    /**
     * @type {?}
     * @private
     */
    NgGxSplitTextDirective.prototype.el;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @record
 */
function Options() { }
if (false) {
    /** @type {?|undefined} */
    Options.prototype.defer;
    /** @type {?|undefined} */
    Options.prototype.onlyWords;
    /** @type {?|undefined} */
    Options.prototype.mask;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NgGxSplitTextModule {
}
NgGxSplitTextModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    NgGxSplitTextDirective,
                ],
                imports: [
                    CommonModule,
                ],
                exports: [
                    NgGxSplitTextDirective,
                ],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NgGxSplitTextDirective, NgGxSplitTextModule, defaultOptions };
//# sourceMappingURL=ng-gx-split-text.js.map
